﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;
using System.Globalization;
using System.Diagnostics;

namespace MinuteConverter
{
    #region frmMain class
    public partial class frmMain : Form
    {
        private DataTable table = null;

        public frmMain()
        {
            InitializeComponent();

            this.grid.Dock = DockStyle.Fill;
            this.grid.InitializeLayout += new InitializeLayoutEventHandler(this.grid_InitializeLayout);
            this.grid.DataSource = this.Data;
        }

        private void grid_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            ColumnsCollection columns = e.Layout.Bands[0].Columns;
            foreach ( UltraGridColumn column in columns )
            {
                string key = column.Key;

                switch ( key )
                {
                    //  Assign the data filter
                    case "Hours":
                        column.Editor.DataFilter = new TimeConversionDataFilter( 2 );
                        break;

                }
            }
        }

        private DataTable Data
        {
            get
            {
                if ( this.table == null )
                {
                    this.table = new DataTable();

                    //  Add columns
                    this.table.Columns.Add( "Hours", typeof(int) );

                    //  Add rows
                    this.table.Rows.Add( new object[]{ 90 } );
                    this.table.Rows.Add( new object[]{ 120 } );
                    this.table.Rows.Add( new object[]{ 60 } );
                    this.table.Rows.Add( new object[]{ 15 } );

                }

                return this.table;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if ( this.grid.ActiveCell != null &&
                 this.grid.ActiveCell.Value is int )
                this.textBox1.Text = this.grid.ActiveCell.Value.ToString();
        }
    }
    #endregion frmMain class

    #region TimeConversionDataFilter class
    /// <summary>
    /// IEditorDataFilter implementation which performs bi-directional
    /// conversions for fractional representations of double values.
    /// </summary>
    public class TimeConversionDataFilter : IEditorDataFilter
    {
        #region Member variables
        
        private int precision = 2;

        #endregion Member variables

        #region Constructors

        /// <summary>
        /// Creates a new instance.
        /// </summary>
        public TimeConversionDataFilter( int precision )
        {
            this.precision = precision > 0 ? precision : 2;
        }

        #endregion Constructors

        #region IEditorDataFilter

        object IEditorDataFilter.Convert(EditorDataFilterConvertArgs conversionArgs)
        {
            object rawValue = conversionArgs.Value;
            ConversionDirection direction = conversionArgs.Direction;

            switch ( direction )
            {
                case ConversionDirection.EditorToOwner:
                {                    
                    conversionArgs.Handled = true; 
                                     
                    double val = 0;
                    if ( rawValue is double )
                        val = (double)rawValue;
                    else
                    if ( rawValue is string )
                    {
                        string stringVal = rawValue as string;
                        double.TryParse( stringVal, out val );
                        return val * 60;
                    }
                }
                break;

                //  Reverse the conversion on the 'EditorToOwner' direction
                case ConversionDirection.OwnerToEditor:
                {                    
                    conversionArgs.Handled = true;

                    if ( rawValue is int )
                    {
                        int val = (int)rawValue;
                        double hours = ((double)val / (double)60);
                        return Math.Round(hours, this.precision);
                    }
                }
                break;
            }

            return conversionArgs.Value;
        }

        #endregion IEditorDataFilter

    }
    #endregion TimeConversionDataFilter class
}
